Scriptname SSX_Cell_Scanner_Quest extends Quest Conditional ; V1.4h

; ((- Properties
SSX_Runner_Quest Property ssxRunnerQuest Auto
Actor Property PlayerRef Auto
SexLabFramework Property SexLab Auto
Faction Property ssxScanningFaction Auto
Faction Property SexLabAnimatingFaction Auto

ObjectReference[] Property beds Auto
int Property numBeds Auto
Actor [] badActors
int badNum = 0

Cell lastScannedCell
Cell currentScannedCell
bool friendsPreviousStatus
bool iAmScanning
; -))

Function doInit()
	friendsPreviousStatus=true
	RegisterForModEvent("ssxStartSSXMod", "startSSXCS")
	RegisterForModEvent("ssxStopSSXMod", "stopSSXCS")
	RegisterForModEvent("ssxStartScanner", "startSSXCS")
	RegisterForModEvent("ssxStopScanner", "stopSSXCS")
	RegisterForModEvent("ssxScanCell", "doItNow")
	ssxRunnerQuest.log("Cell Scanner events registered", SSX_Runner_Quest.notifScanDetails())
	iAmScanning=false
	badActors = new Actor[100]
EndFunction

; ((- Generic no-state events
Event startSSXCS(int mode)
	if mode==0
		ssxRunnerQuest.log("Cell Scanner going to active state", SSX_Runner_Quest.notifScanDetails())
		goToState("active")
	endIf
EndEvent

Event stopSSXCS(String unused)
	if ssxRunnerQuest.scanMode!=0
		return
	endIf
	ssxRunnerQuest.log("Cell Scanner going to inactive state", SSX_Runner_Quest.notifScanDetails())
	if getState()=="active"
		goToState("inactive")
	endIf
	iAmScanning=false
EndEvent

Event doItNow(String mod)
	if mod=="CellScanner" && ssxRunnerQuest.iAmActive && getState()!="active"
		UnregisterForUpdate()
		RegisterForSingleUpdate(2.0)
		goToState("active")
	endIf
EndEvent

Function cleanup()
	while badNum
		badNum -= 1
		badActors[badNum].removeFromFaction(ssxScanningFaction)
		badActors[badNum] = none
	endWhile
	ssxRunnerQuest.removeFromScanningFaction()
	ssxRunnerQuest.schedulerIAmDone("Scanner")
	iAmScanning=false
EndFunction

bool Function reserveSchedule()
	int s = ssxRunnerQuest.schedulerCanIDo("Scanner")
	while s
			if s == 2
					return false
			endIf ; s == 1
			Utility.waitMenuMode(0.5)
			s = ssxRunnerQuest.schedulerCanIDo("Scanner")
	endWhile ; s == 0
	cleanup()
	gotoState("inactive")
	return true
EndFunction

; -))


state inactive
	Function OnBeginState()
		{Be sure we stop everything}
		UnregisterForUpdate()
		Utility.waitMenuMode(1.0)
		UnregisterForUpdate()
		cleanup()
	EndFunction
	
	Event startSSXCS(int mode)
		if mode==0
			ssxRunnerQuest.log("Cell Scanner going to active state", SSX_Runner_Quest.notifScanDetails())
			goToState("active")
		endIf
	EndEvent

	Event stopSSXCS(String unused)
		return
	EndEvent

	Event doItNow(String mod)
		ssxRunnerQuest.log("Cannot do an update, I am stopped: Cell Scanner", SSX_Runner_Quest.notifScanDetails())
	EndEvent	
endState

state active
	Function OnBeginState()
		if ssxRunnerQuest.scanMode!=0
			ssxRunnerQuest.log("The scan mode is not set to Cell, going to inactive state", SSX_Runner_Quest.notifScanDetails())
			GoToState("inactive")
			return
		endIf
	
		RegisterForModEvent("ssxStartSSXMod", "startSSXCS")
		RegisterForModEvent("ssxStopSSXMod", "stopSSXCS")
		RegisterForModEvent("ssxScanCell", "doItNow")
		ssxRunnerQuest.updateCellAndLocation(PlayerRef.getParentCell())
		if ssxRunnerQuest.startRightAway==1
			RegisterForSingleUpdate(1.0)
			ssxRunnerQuest.log("Starting right away: Cell Scanner", SSX_Runner_Quest.notifScanDetails())
		else
			RegisterForSingleUpdate(ssxRunnerQuest.rescanCellTime)
			ssxRunnerQuest.log("Cell Scanner: Time to wait for the next update = " + ssxRunnerQuest.rescanCellTime + " seconds", SSX_Runner_Quest.notifScanDetails())
		endIf
		friendsPreviousStatus=true
	EndFunction

	Event startSSXCS(int mode)
		if mode!=0
			GoToState("inactive")
		endIf
	EndEvent

	Event stopSSXCS(String unused)
		if ssxRunnerQuest.scanMode==0
			ssxRunnerQuest.log("Cell Scanner going to inactive state", SSX_Runner_Quest.notifScanDetails())
		endIf
		UnregisterForUpdate()
		GoToState("inactive")
	EndEvent

	Event doItNow(String mod)
		if mod=="CellScanner" && !iAmScanning
			UnregisterForUpdate()
			RegisterForSingleUpdate(0.5)
		endIf
	EndEvent
	
	Event OnUpdate()
		if reserveSchedule()
			return
		endIf
		ssxRunnerQuest.log("Cell Scanner Checker - Starting the scan", SSX_Runner_Quest.notifScanDetails())
		iAmScanning=true
		ssxRunnerQuest.clearCached() ; wipe all cached data for actors
		ssxRunnerQuest.playerSex = 0
		if (ssxRunnerQuest.genderMode==0 && PlayerRef.getLeveledActorBase().getSex()) || (ssxRunnerQuest.genderMode!=0 && Math.LogicalAnd(SexLab.getGender(PlayerRef), 1))
			ssxRunnerQuest.playerSex = 1
		endIf
		
		if reserveSchedule()
			return
		endIf
		
		; remove actors that are no longer valid
		ssxRunnerQuest.removeNoMoreValidActors()

		if reserveSchedule()
			return
		endIf

		; scan the player cell, updating also the actor statuses
		scanThePlayerCell()
		
		if reserveSchedule()
			return
		endIf
		
		; Update the ssxRunnerQuest property, but check if at least two guys are reuired
		if ssxRunnerQuest.needP && (ssxRunnerQuest.needM || ssxRunnerQuest.needF || ssxRunnerQuest.needC || ssxRunnerQuest.needH || ssxRunnerQuest.needW || ssxRunnerQuest.needN) && ssxRunnerQuest.noActors()
			; Not enough actors
			while ssxRunnerQuest.letsWait
				Utility.waitMenuMode(1.0)
			endWhile
			RegisterForSingleUpdate(ssxRunnerQuest.rescanCellTime)
			ssxRunnerQuest.log("Cell Scanner - Scan completed. Not enough actors", SSX_Runner_Quest.notifScanDetails())
			cleanup()
			return
		endIf
		
		if reserveSchedule()
			return
		endIf

		; Restart the Rule Checker in case this was out first scan
		if !ssxRunnerQuest.noActors() && ssxRunnerQuest.checkingMode==0
			int handle = ModEvent.Create("ssxStartSSXRC")
			ModEvent.PushInt(handle, 0)
			ModEvent.Send(handle)
		elseIf ssxRunnerQuest.checkingMode==1
			int handle = ModEvent.Create("ssxDoIt")
			ModEvent.PushString(handle, "RuleChecker")
			ModEvent.Send(handle)
		endIf

		while ssxRunnerQuest.letsWait
			Utility.waitMenuMode(1.0)
		endWhile
		
		; Find beds
		numBeds = ssxRunnerQuest.scanBeds(beds)
		ssxRunnerQuest.setBeds(beds, numBeds)

		if reserveSchedule()
			return
		endIf
		
		if ssxRunnerQuest.getDebug(SSX_Runner_Quest.notifActorsFoundLong())
			ssxRunnerQuest.dumpAllActors("Cell Scan")
		endIf
		
		; then wait for the next cycle
		RegisterForSingleUpdate(ssxRunnerQuest.rescanCellTime)
		ssxRunnerQuest.log("Cell Scanner - Scan completed. Found " + ssxRunnerQuest.countActors() + " actors", SSX_Runner_Quest.notifScanDetails())
		cleanup()
	EndEvent
endState





Function scanThePlayerCell()
{Scans the whole player cell and creates an array of actors and their status}
	Cell c = PlayerRef.getParentCell()
	currentScannedCell = c
	ssxRunnerQuest.startT("Cell scanning")
	Actor[] inCell = MiscUtil.ScanCellNPCs(PlayerRef, 0.0)
	
	badNum = 0
	int num = inCell.length
	while num
		num -= 1
		Actor a = inCell[num]
		if a
			if ssxRunnerQuest.checkValidityForScan(a)
				ssxRunnerQuest.addToAllActors(a)
			else
				if badNum > 99
					badNum = 99
				endIf
				badActors[badNum] = a
				badNum += 1						
				If !a.isInFaction(SexLabAnimatingFaction)
					int n = ssxRunnerQuest.searchScene(none, -1, a)
					if n!=-1
						ssxRunnerQuest.releaseScene(none, n, none)
					endIf
				endIf
			endIf
			if !a.IsInFaction(ssxScanningFaction)
				a.addToFaction(ssxScanningFaction)
			endIf
		endIf
		if reserveSchedule()
			return
		endIf
		while ssxRunnerQuest.letsWait
			Utility.waitMenuMode(1.0)
		endWhile
	endWhile
	
	lastScannedCell = currentScannedCell

	ssxRunnerQuest.endT("Cell scanning")
EndFunction

Function scanThePlayerCellOLD()
{Scans the whole player cell and creates an array of actors and their status}
	Cell c = PlayerRef.getParentCell()
	currentScannedCell = c
	
	ssxRunnerQuest.startT("Cell scanning")
	badNum = 0
	int num = (c.GetNumRefs(62)) as Int
	while num
		num -= 1
		Actor a = c.GetNthRef(num, 62) as Actor
		if a
			if ssxRunnerQuest.checkValidityForScan(a)
				ssxRunnerQuest.addToAllActors(a)
			else
				if badNum > 99
					badNum = 99
				endIf
				badActors[badNum] = a
				badNum += 1						
				If !a.isInFaction(SexLabAnimatingFaction)
					int n = ssxRunnerQuest.searchScene(none, -1, a)
					if n!=-1
						ssxRunnerQuest.log("## Release scene with " + a.getDisplayName(), SSX_Runner_Quest.notifScanDetails())
						ssxRunnerQuest.releaseScene(none, n, none)
					endIf
				endIf
			endIf
			if !a.IsInFaction(ssxScanningFaction)
				a.addToFaction(ssxScanningFaction)
			endIf
		endIf
		if reserveSchedule()
			return
		endIf
		while ssxRunnerQuest.letsWait
			Utility.waitMenuMode(1.0)
		endWhile
	endWhile
	
	lastScannedCell = currentScannedCell

	ssxRunnerQuest.endT("Cell scanning")
EndFunction
