Scriptname SSX_Rule_Player_Quest extends Quest ; 1.4h
{Grabs all the valid rules, select one, fill it, and runs it if possible}


; ((- Properties
SSX_Runner_Quest Property ssxRunnerQuest Auto
Actor Property PlayerRef Auto
SSX_Rule_Alias[] Property possibleRules Auto
Idle Property WakeUp Auto
Faction Property ssxCreatures Auto
Faction Property ssxAllFriends Auto
Faction Property ssxInvolved Auto

int localStartMode = 0
Actor localParticipant1 = None
Actor localParticipant2 = None
int localPrevRule = -1
int localRuleIndex = -1

; -))




Function doInit()
	RegisterForModEvent("ssxStartSSXMod", "startSSXRP")
	RegisterForModEvent("ssxStopSSXMod", "stopSSXRP")
	RegisterForModEvent("ssxPlayRules", "playRules") ; Pick a random rule from the valid ones and plays it
	RegisterForModEvent("ssxPlayRulesNow", "playRule") ; Called by the Rule Checker after an EventCatchers, with all the IDs of the rules (4 max, right now, they may be 0)
	ssxRunnerQuest.log("Rule Player activated and events registered", SSX_Runner_Quest.notifPlayerDetails())
endFunction

; ((- Generic no-state events
Event startSSXRP(int unused)
	ssxRunnerQuest.log("Rule Player going to active state", SSX_Runner_Quest.notifPlayerDetails())
	goToState("active")
EndEvent

Event stopSSXRP(String unused)
	ssxRunnerQuest.log("Rule Player going to inactive state", SSX_Runner_Quest.notifPlayerDetails())
	goToState("inactive")
EndEvent

Event playRules(int unused)
	ssxRunnerQuest.log("Rule Player: got a play rules in empty state", SSX_Runner_Quest.notifPlayerDetails())
EndEvent

Event playRule(Form a, Form b, int catchMode, int r1, int r2, int r3, int r4, int r5, int r6, int r7, int r8)
	ssxRunnerQuest.stopCeaseFire(catchMode)
	ssxRunnerQuest.log("Rule Player: got a play rules in empty state", SSX_Runner_Quest.notifErrors())
EndEvent

SSX_Rule_Alias Function pickRule(Int prevRuleID)

endFunction

bool Function reserveSchedule()
	int s = ssxRunnerQuest.schedulerCanIDo("Rule Player")
	while s
			if s == 2
					return false
			endIf ; s == 1
			Utility.waitMenuMode(0.2)
			s = ssxRunnerQuest.schedulerCanIDo("Rule Player")
	endWhile ; s == 0
	cleanup()
	gotoState("inactive")
	return true
endFunction

Function cleanup()
	ssxRunnerQuest.schedulerIAmDone("Rule Player")
endFunction

Function redo()
	UnregisterForUpdate()
	cleanup()
	RegisterForSingleUpdate(5.0)
endFunction

; -))

state inactive
	Function OnBeginState()
		{Be sure we stop everything}
		UnregisterForUpdate()
		Utility.waitMenuMode(0.2)
		UnregisterForUpdate()
	endFunction

	Event startSSXRP(int unused)
		goToState("active")
	EndEvent

	Event stopSSXRP(String unused)
	EndEvent
endState

state active
	Function OnBeginState()
		{Set up everything including the cycle}
		localRuleIndex = -1
		localStartMode = 0
		localParticipant1 = None
		localParticipant2 = None
		localPrevRule = -1
		ssxRunnerQuest.log("Started", SSX_Runner_Quest.notifActivationN())
	endFunction

	Event startSSXRP(int unused)
		ssxRunnerQuest.log("I am already active: Rule Player", SSX_Runner_Quest.notifPlayerDetails())
	EndEvent

	Event stopSSXRP(String unused)
		UnregisterForUpdate()
		goToState("inactive")
	EndEvent

	Event OnUpdate()
		if localPrevRule!=-1
			playRules(localPrevRule)
			localPrevRule=-1
		else
			playRules(0)
		endIf
	EndEvent


	Event playRules(Int prevRuleID)
debug.trace("SSX SSX SSX received event playrules")
		if reserveSchedule()
			return
		endIf
		
		ssxRunnerQuest.log("Rule Player: playing rules", SSX_Runner_Quest.notifPlayerDetails())

		if ssxRunnerQuest.areMenuOpen>0 || ssxRunnerQuest.checkMenuOpen()
			if prevRuleID
				localPrevRule=prevRuleID
			endIf
			redo()
			ssxRunnerQuest.log("Rule Player: Restarting in 5 seconds because a menu is open", SSX_Runner_Quest.notifPlayerDetails())
			return
		endIf

		; There is space for another scene?
		if ssxRunnerQuest.countScenesPlaying() >= ssxRunnerQuest.maxNumberOfParallelScenes
			ssxRunnerQuest.log("Rule Player: Too many scenes already playing", SSX_Runner_Quest.notifPlayerDetails())
			cleanup()
			return
		endIf

		; Pick random rule out of valid rules
		int maxPrio = 9999 ; highest priority is lowest number
		int totProb = 0 ; collect probabilities
		int i = ssxRunnerQuest.rules.length
		SSX_Rule_Alias theRule = none
		while i
			i -= 1
			SSX_Rule_Alias r = ssxRunnerQuest.rules[i]
			if r.inUse == 2 &&  ((r.status == 2 && r.startsOn == 0 && prevRuleID == 0) || ( r.status > 0 && r.startsOn == 2 && r.prevRule == prevRuleID ) )
				int prio = r.probability
				if ssxRunnerQuest.priorityMode && prio < maxPrio
					maxPrio = prio
					totProb = 0
				endIf
				if ssxRunnerQuest.priorityMode == 0 || prio == maxPrio
					int prob = Math.pow(2, 4 - prio) as Int
					totProb += prob
					if Utility.randomInt(1, totProb) <= prob && (r.timeBeforeReDo==0.0 || r.lastRunTime==0.0 || (Utility.GetCurrentGameTime()-r.lastRunTime)*3600.0>=r.timeBeforeReDo)
						theRule = r
					endIf
				endIf
			endIf
		endWhile

		if ssxRunnerQuest.areMenuOpen>0 || ssxRunnerQuest.checkMenuOpen()
			if prevRuleID
				localPrevRule=prevRuleID
			endIf
			redo()
			ssxRunnerQuest.log("Rule Player: Restarting in 5 seconds because a menu is open", SSX_Runner_Quest.notifPlayerDetails())
			return
		endIf
		
		if reserveSchedule()
			return
		endIf

		while ssxRunnerQuest.letsWait
			Utility.waitMenuMode(1.0)
		endWhile

		if !theRule
			; Nothing found
			cleanup()
			ssxRunnerQuest.log("Rule Player: No applicable rules found.", SSX_Runner_Quest.notifErrors())
			int handle = ModEvent.Create("ssxDoIt")
			ModEvent.PushString(handle, "RuleChecker")
			ModEvent.Send(handle)
			return
		endIf
		
		; Fill the rule and start it
		if !fillTheRule(theRule, 0)
			cleanup()
			ssxRunnerQuest.log("The rule \"" + theRule.name + "\" cannot start.", SSX_Runner_Quest.notifBadRule())
			int handle = ModEvent.Create("ssxDoIt")
			ModEvent.PushString(handle, "RuleChecker")
			ModEvent.Send(handle)
			return
		endIf
		
		if reserveSchedule()
			return
		endIf

		while ssxRunnerQuest.letsWait
			Utility.waitMenuMode(1.0)
		endWhile
		
		; Check if orgies are enabled, in this case we can restart it with other actors
		if theRule.enableOrgies && ssxRunnerQuest.countScenesPlaying()<ssxRunnerQuest.scenes.length - 2
			if reserveSchedule()
				return
			endIf
			while fillTheRule(theRule, 0)
				if reserveSchedule()
					return
				endIf
			endWhile
			cleanup()
		endIf
		cleanup()
	EndEvent

	Event playRule(Form a, Form b, int catchMode, int r1, int r2, int r3, int r4, int r5, int r6, int r7, int r8)
		if reserveSchedule()
			return
		endIf
		
		; find which rule can be filled for real, with the specified actors (b may be empty)
		; catchMode: 0=unspecified (random), 1=player ask (not really an event), 2=nudity, 3=health, 4=combat, 5=bleedout, 6=sleep
		ssxRunnerQuest.log("Rule Player: playing rules from Event Catchers", SSX_Runner_Quest.notifPlayerDetails())

		; There is space for another scene?
		if ssxRunnerQuest.countScenesPlaying() >= ssxRunnerQuest.maxNumberOfParallelScenes
			ssxRunnerQuest.log("Rule Player: Too many scenes already playing", SSX_Runner_Quest.notifPlayerDetails())
			cleanup()
			return
		endIf

		if reserveSchedule()
			return
		endIf

		while ssxRunnerQuest.letsWait
			Utility.waitMenuMode(1.0)
		endWhile
		
		SSX_Rule_Alias[] possibleRulesLocal = new SSX_Rule_Alias[8]

		int numPossibleRules = possibleRulesLocal.length
		while numPossibleRules
			numPossibleRules-=1
			possibleRulesLocal[numPossibleRules]=none
		endWhile

		if reserveSchedule()
			return
		endIf
		
		SSX_Rule_Alias r
		if r1!=0
			r = ssxRunnerQuest.getRuleById(r1)
			if r
				possibleRulesLocal[numPossibleRules]=r
				numPossibleRules+=1
			endIf
		endIf
		if r2!=0
			r = ssxRunnerQuest.getRuleById(r2)
			if r
				possibleRulesLocal[numPossibleRules]=r
				numPossibleRules+=1
			endIf
		endIf
		if r3!=0
			r = ssxRunnerQuest.getRuleById(r3)
			if r
				possibleRulesLocal[numPossibleRules]=r
				numPossibleRules+=1
			endIf
		endIf
		if r4!=0
			r = ssxRunnerQuest.getRuleById(r4)
			if r
				possibleRulesLocal[numPossibleRules]=r
				numPossibleRules+=1
			endIf
		endIf
		if r5!=0
			r = ssxRunnerQuest.getRuleById(r5)
			if r
				possibleRulesLocal[numPossibleRules]=r
				numPossibleRules+=1
			endIf
		endIf
		if r6!=0
			r = ssxRunnerQuest.getRuleById(r6)
			if r
				possibleRulesLocal[numPossibleRules]=r
				numPossibleRules+=1
			endIf
		endIf
		if r7!=0
			r = ssxRunnerQuest.getRuleById(r7)
			if r
				possibleRulesLocal[numPossibleRules]=r
				numPossibleRules+=1
			endIf
		endIf
		if r8!=0
			r = ssxRunnerQuest.getRuleById(r8)
			if r
				possibleRulesLocal[numPossibleRules]=r
				numPossibleRules+=1
			endIf
		endIf
		
		if reserveSchedule()
			return
		endIf

		while ssxRunnerQuest.letsWait
			Utility.waitMenuMode(1.0)
		endWhile
		
		if numPossibleRules==0
			; Find a rule by calling the Rule Checker
			ssxRunnerQuest.log("Rule Player: No rules found from the catcher, asking Rule Checker.", SSX_Runner_Quest.notifPlayerDetails())
			ssxRunnerQuest.stopCeaseFire(catchMode)
			if ssxRunnerQuest.checkingMode==0
				int handle = ModEvent.Create("ssxStartSSXRC")
				ModEvent.PushInt(handle, 0)
				ModEvent.Send(handle)
			else
				int handle = ModEvent.Create("ssxDoIt")
				ModEvent.PushString(handle, "RuleChecker")
				ModEvent.Send(handle)
			endIf
			cleanup()
			return
		endIf

		if reserveSchedule()
			return
		endIf

		while ssxRunnerQuest.letsWait
			Utility.waitMenuMode(1.0)
		endWhile
		
		; Pick random rule out of valid rules
		Actor p1 = a as Actor
		Actor p2 = b as Actor

		int maxPrio = 9999 ; highest priority is lowest number
		int totProb = 0 ; collect probabilities
		int i = possibleRulesLocal.length
		bool oneValid = false
		SSX_Rule_Alias theRule = none
		while i
			i -= 1
			r = possibleRulesLocal[i]
			if r && r.inUse == 2 && r.status == 2 && r.startsOn == 0
				bool isValid = r.validActor(p2)
				if isValid
					if !oneValid
						oneValid = true
						maxPrio = 9999
						totProb = 0
					endIf
				endIf
				if isValid || (!isValid && !oneValid)
					int prio = r.probability
				
					if ssxRunnerQuest.priorityMode && prio < maxPrio
						maxPrio = prio
						totProb = 0
					endIf
					if ssxRunnerQuest.priorityMode == 0 || prio == maxPrio
						int prob = Math.pow(2, 4 - prio) as Int
						totProb += prob
						if Utility.randomInt(1, totProb) <= prob
							theRule = r
							ssxRunnerQuest.Log("Rule is " + r.name, SSX_Runner_Quest.notifPlayerDetails())
						endIf
					endIf
				endIf
			endIf
		endWhile
		if !oneValid
			p2 = None
		endIf
		
		if reserveSchedule()
			return
		endIf

		while ssxRunnerQuest.letsWait
			Utility.waitMenuMode(1.0)
		endWhile

		if !theRule
			; Nothing found
			ssxRunnerQuest.log("Rule Player: No applicable rules found.", SSX_Runner_Quest.notifErrors())
			ssxRunnerQuest.stopCeaseFire(catchMode)
			cleanup()
			int handle = ModEvent.Create("ssxDoIt")
			ModEvent.PushString(handle, "RuleChecker")
			ModEvent.Send(handle)
			return
		endIf
		
		; Fill the rule and start it
		if !fillTheRule(theRule, catchMode, p1, p2)
			ssxRunnerQuest.log("The rule \"" + theRule.name + "\" cannot start.", SSX_Runner_Quest.notifBadRule())
			ssxRunnerQuest.stopCeaseFire(catchMode)
			return
		endIf
		; If the rule is filled (if we are here it is) and the catchMode==4 (combat) let's evaluate if we need to stop combats
		if catchMode==4 ; Combat
			if theRule.stopAllCombats==2 ; Always
				ssxRunnerQuest.startCeaseFire(catchMode)
			elseIf theRule.stopAllCombats==1 ; player/followers only
				if (p1 && (p1==PlayerRef || p1.isPlayerTeammate())) || (p2 && (p2==PlayerRef || p2.isPlayerTeammate()))
					ssxRunnerQuest.startCeaseFire(catchMode)
				endIf
			endIf
		endIf
		
		if reserveSchedule()
			return
		endIf

		while ssxRunnerQuest.letsWait
			Utility.waitMenuMode(1.0)
		endWhile
		
		; Check if orgies are enabled, in this case we can restart it with other actors
		if theRule.enableOrgies && ssxRunnerQuest.countScenesPlaying()<ssxRunnerQuest.scenes.length - 2
			fillTheRule(theRule, catchMode)
		endIf
		cleanup()
	EndEvent

endState

bool Function fillTheRule(SSX_Rule_Alias r, int catchMode, Actor preferred1 = None, Actor preferred2 = None)
{catchMode: 0=unspecified (random), 1=player ask (not really an event), 2=nudity, 3=health, 4=combat, 5=bleedout, 6=sleep}
	Actor chosenD = None
	Actor chosenS = None
	Actor chosenC1 = None
	Actor chosenC2 = None
	Actor chosenC3 = None
	ObjectReference chosenB = None

	; Quick player check
	int playerPos = -1
	if r.isPlayer(0)
		playerPos=0
	elseIf r.isPlayer(1)
		playerPos=1
	elseIf r.isPlayer(2)
		playerPos=2
	endIf
	if playerPos>-1 && (PlayerRef.IsInFaction(ssxRunnerQuest.ssxInvolved))
		int i = 5
		while (!ssxRunnerQuest.validateActor(PlayerRef)) && i > 0
			i -= 1
			Utility.waitMenuMode(1.0)
			if reserveSchedule()
				return false
			endIf
		endWhile
		if i == 0
			ssxRunnerQuest.log("The player cannot be filled for " + r.name + " because they're already involved", SSX_Runner_Quest.notifBadRule())
			return false
		endIf
	endIf
	
		
	; Get the filtered actors depending on Races and Sneaking, Weapon, Combat, etc.
	int dstActorsLen = r.getValidActorsLen(0)
	int srcActorsLen = r.getValidActorsLen(1)
	int collActorsLen = r.getValidActorsLen(2)
	Actor[] dstActors = r.validDst
	Actor[] srcActors = r.validSrc
	Actor[] collActors = r.validColl
	bool notReallyRequiredD = r.doNotInvolveDst()
	bool notReallyRequiredS = r.doNotInvolveSrc()
	bool notReallyRequiredC = r.doNotInvolveColl()
	Actor[] toAvoids = new Actor[4]

	if reserveSchedule()
		return false
	endIf
		
	; Check the target actor
	if r.isPlayer(0) && PlayerRef.IsInFaction(ssxRunnerQuest.SexLabAnimatingFaction)
		ssxRunnerQuest.log("The Target actor is the Player and cannot be filled: " + r.name, SSX_Runner_Quest.notifBadRule())
		return false
	endIf
	if dstActorsLen==0 && !r.isPlayer(0)
		ssxRunnerQuest.log("The Target actor cannot be filled: " + r.name, SSX_Runner_Quest.notifBadRule())
		return false
	endIf
	; Check the source actor
	if r.isPlayer(1) && PlayerRef.IsInFaction(ssxRunnerQuest.SexLabAnimatingFaction)
		ssxRunnerQuest.log("The Source actor is the Player and cannot be filled: " + r.name, SSX_Runner_Quest.notifBadRule())
		return false
	endIf
	if srcActorsLen==0 && !r.isPlayer(1) && !r.isNobody(1)
		ssxRunnerQuest.log("The Source actor cannot be filled: " + r.name, SSX_Runner_Quest.notifBadRule())
		return false
	endIf
	; Check the collaborator actor
	if r.isPlayer(2) && PlayerRef.IsInFaction(ssxRunnerQuest.SexLabAnimatingFaction)
		ssxRunnerQuest.log("The Collaborator is the Player and cannot be filled: " + r.name, SSX_Runner_Quest.notifBadRule())
		return false
	endIf
	if collActorsLen==0 && !r.isPlayer(2) && !r.isNobody(2)
		ssxRunnerQuest.log("The Collaborator cannot be filled: " + r.name, SSX_Runner_Quest.notifBadRule())
		return false
	endIf

	if reserveSchedule()
		return false
	endIf
		
	; Who has to be the preferred? Avoid the ones already involved...
	if preferred1 && ((dstActors.find(preferred1)==-1 && srcActors.find(preferred1)==-1) || preferred1.isInFaction(ssxInvolved))
		preferred1=none
	endIf
	if preferred2 && ((dstActors.find(preferred2)==-1 && srcActors.find(preferred2)==-1) || preferred2.isInFaction(ssxInvolved))
		preferred2=none
	endIf

	; catchMode: 0=unspecified (random), 1=player ask (not really an event), 2=nudity, 3=health, 4=combat, 5=bleedout, 6=sleep, 7=clothing
	if catchMode==1 || catchMode==11; In case PlayerAsks:	pref1=Player 	pref2=speaker
		; player active or passive?
		if r.isPlayer(1) || r.isGenericWPl(1)
			chosenS=PlayerRef
			if preferred1==PlayerRef
				chosenD=preferred2
			else
				chosenD=preferred1
			endIf
		elseIf r.isPlayer(0) || r.isGenericWPl(0)
			chosenD=PlayerRef
			if preferred1==PlayerRef
				chosenS=preferred2
			else
				chosenS=preferred1
			endIf
		else
			chosenD=preferred1
			chosenS=preferred2
		endIf
	elseIf catchMode==2 ; Nudity, the preferred1 is the one that is nude
		if r.isStartingOn(1, dstP=0)
			chosenD=preferred1
			addToAvoids(toAvoids, preferred1)
		elseIf r.isStartingOn(1, srcP=0)
			chosenS=preferred1
			addToAvoids(toAvoids, preferred1)
		elseIf r.isStartingOn(1, collP=0)
			chosenC1=preferred1
			addToAvoids(toAvoids, preferred1)
		endIf
	
	elseIf catchMode==3 ; Health
		if r.isStartingOn(5, dstP=0)
			chosenD=preferred1
			addToAvoids(toAvoids, preferred1)
			chosens=preferred2
			addToAvoids(toAvoids, preferred2)
		elseIf r.isStartingOn(5, srcP=0)
			chosenS=preferred1
			addToAvoids(toAvoids, preferred1)
			chosenD=preferred2
			addToAvoids(toAvoids, preferred2)
		elseIf r.isStartingOn(5, collP=0)
			chosenS=preferred2
			addToAvoids(toAvoids, preferred2)
			chosenC1=preferred1
			addToAvoids(toAvoids, preferred1)
		endIf
		
	elseIf catchMode==4 ; Combat
		if r.isStartingOn(11, dstP=0)
			chosenD=preferred1
			addToAvoids(toAvoids, preferred1)
			chosenS=preferred2
			addToAvoids(toAvoids, preferred2)
		elseIf r.isStartingOn(11, srcP=0)
			chosenS=preferred1
			addToAvoids(toAvoids, preferred1)
			chosenD=preferred2
			addToAvoids(toAvoids, preferred2)
		elseIf r.isStartingOn(11, collP=0)
			chosenS=preferred2
			addToAvoids(toAvoids, preferred2)
			chosenC1=preferred1
			addToAvoids(toAvoids, preferred1)
		endIf
		
	elseIf catchMode==5 ; Bleedout
		if r.isStartingOn(12, dstP=0)
			chosenD=preferred1
			addToAvoids(toAvoids, preferred1)
			chosenS=preferred2
			addToAvoids(toAvoids, preferred2)
		elseIf r.isStartingOn(12, srcP=0)
			chosenS=preferred1
			addToAvoids(toAvoids, preferred1)
			chosenD=preferred2
			addToAvoids(toAvoids, preferred2)
		elseIf r.isStartingOn(12, collP=0)
			chosenS=preferred2
			addToAvoids(toAvoids, preferred2)
			chosenC1=preferred1
			addToAvoids(toAvoids, preferred1)
		endIf

	elseIf catchMode==6 ; Sleep
		if r.isStartingOn(8, dstP=0)
			chosenD=preferred1
			addToAvoids(toAvoids, preferred1)
		elseIf r.isStartingOn(8, srcP=0)
			chosenS=preferred1
			addToAvoids(toAvoids, preferred1)
		elseIf r.isStartingOn(8, collP=0)
			chosenC1=preferred1
			addToAvoids(toAvoids, preferred1)
		endIf
	endIf

	if reserveSchedule()
		return false
	endIf
	
	; Now fill the actors
	if !ssxRunnerQuest.validateActor(chosenS) && !r.isNobody(1) ; Need to find the Source
		chosenS = fillTheActor(r, 1, srcActors, srcActorsLen, notReallyRequiredS, toAvoids)
		if !chosenS
			ssxRunnerQuest.log("The Source actor cannot be found (" + ssxRunnerQuest.getPositionShortName(r, 1) + "): " + r.name, SSX_Runner_Quest.notifBadRule())
			return false
		endIf
	endIf
	addToAvoids(toAvoids, chosenS)
	if !ssxRunnerQuest.validateActor(chosenD) ; Need to find the Target
		chosenD = fillTheActor(r, 0, dstActors, dstActorsLen, notReallyRequiredD, toAvoids, none, ssxRunnerQuest.getMFSB(chosenS))
		if !chosenD
			ssxRunnerQuest.log("The Target actor cannot be found (" + ssxRunnerQuest.getPositionShortName(r, 0) + "): " + r.name, SSX_Runner_Quest.notifBadRule())
			ssxRunnerQuest.log("Source is " + chosenS.getDisplayName() + " and the its sexual orientation is " +  ssxRunnerQuest.orientationModesC[ssxRunnerQuest.getSexBitsForType(1)], SSX_Runner_Quest.notifBadRule())
			return false
		endIf
	endIf
	addToAvoids(toAvoids, chosenD)
		
	; ((- Fill the actors, check if we need it for real inside getRndActor, and don't fail if it is not required
	
	; OK, here we may decide if the collaborators are optional
	
	if !r.isNobody(2) && !ssxRunnerQuest.validateActor(chosenC1)
		if ssxRunnerQuest.validateActor(preferred2) && toAvoids.find(preferred2) == -1
			chosenC1 = preferred2
			addToAvoids(toAvoids, preferred2)
		else
			chosenC1 = fillTheActor(r, 2, collActors, collActorsLen, notReallyRequiredC, toAvoids, forTheRace=chosenS)
		endIf
		if !chosenC1 && !r.getOptionalCollaborators()
			ssxRunnerQuest.log("The Collaborator cannot be found (" + ssxRunnerQuest.getPositionShortName(r, 2) + "): " + r.name, SSX_Runner_Quest.notifBadRule())
			return false
		endIf
		addToAvoids(toAvoids, chosenC1)
	else
		chosenC1 = None
	endIf
	if !r.isNobody(2) && r.getNumColls()>1 && chosenC1
		chosenC2 = fillTheActor(r, 2, collActors, collActorsLen, notReallyRequiredC, toAvoids, forTheRace=chosenS)
		if !chosenC2 && !r.getOptionalCollaborators()
			ssxRunnerQuest.log("The second Collaborator cannot be found (" + ssxRunnerQuest.getPositionShortName(r, 2) + "): " + r.name, SSX_Runner_Quest.notifBadRule())
			return false
		endIf
		addToAvoids(toAvoids, chosenC2)
	else
		chosenC2 = None
	endIf
	if !r.isNobody(2) && r.getNumColls()>2 && chosenC2
		chosenC3 = fillTheActor(r, 2, collActors, collActorsLen, notReallyRequiredC, toAvoids, forTheRace=chosenS)
		if !chosenC3 && !r.getOptionalCollaborators()
			ssxRunnerQuest.log("The third Collaborator cannot be found (" + ssxRunnerQuest.getPositionShortName(r, 2) + "): " + r.name, SSX_Runner_Quest.notifBadRule())
			return false
		endIf
	else
		chosenC3 = None
	endIf
	; -))

	if reserveSchedule()
		return false
	endIf

	if r.isBedRequired()
		if r.isPlayerInvolved()
			chosenB = ssxRunnerQuest.getClosestAvailableBed(playerRef)
		else
			chosenB = ssxRunnerQuest.getClosestAvailableBed(chosenD)
		endIf
	endIf
	if r.isBedMandatory() && !chosenB
		ssxRunnerQuest.log("A mandatory bed cannot be found", SSX_Runner_Quest.notifWarnings())
		return false
	endIf
	
	if reserveSchedule()
		return false
	endIf
		
	; Finalize who is really required and who is not
	if !notReallyRequiredD && chosenD
		chosenD.addToFaction(ssxInvolved)
	endIf
	if !notReallyRequiredS && chosenS
		chosenS.addToFaction(ssxInvolved)
	endIf
	if !notReallyRequiredC
		if chosenC1
			chosenC1.addToFaction(ssxInvolved)
		endIf
		if chosenC2
			chosenC2.addToFaction(ssxInvolved)
		endIf
		if chosenC3
			chosenC3.addToFaction(ssxInvolved)
		endIf
	endIf
	r.status = 3 ; In use
	; Send the mod message to the Scene Manager to start the scene
	int handle = ModEvent.Create("ssxStartScene")
	ModEvent.PushInt(handle, ssxRunnerQuest.getRuleIndex(r))
	ModEvent.PushForm(handle, chosenD)
	ModEvent.PushForm(handle, chosenS)
	ModEvent.PushForm(handle, chosenC1)
	ModEvent.PushForm(handle, chosenC2)
	ModEvent.PushForm(handle, chosenC3)
	ModEvent.PushForm(handle, chosenB)
	if catchMode!=1 && catchMode != 11
		catchMode = 0
	endIf
	ModEvent.PushInt(handle, catchMode)
	ModEvent.Send(handle)
	
	return true
endFunction

Function addToAvoids(Actor[] avoids, Actor avoid)
	if !avoid || !avoids
		return
	endIf
	if avoids.find(avoid) != -1
		return
	endIf
	int i = avoids.find(none)
	if i != -1
		avoids[i] = avoid
	endIf
	return
endFunction
	
; otherGender: 0=Male Human (M), 1=Female Human (F), 2=Male Creature (S), 3=Female Creature (B)
Actor Function fillTheActor(SSX_Rule_Alias r, int pos, Actor[] people, int len, bool notReallyRequired, Actor[] avoids, Actor forTheRace=None, int otherGender=-1)
	{Player case}
	if r.isPlayer(pos) && (notReallyRequired || !PlayerRef.isInFaction(ssxRunnerQuest.SexLabAnimatingFaction)) && avoids.find(PlayerRef)==-1 && !PlayerRef.isInFaction(ssxRunnerQuest.ssxInvolved)
		if notReallyRequired
			return PlayerRef ; Always good if not required
		endIf
		if ssxRunnerQuest.excludeActorsWithCum==1 && ssxRunnerQuest.SexLab.CountCum(PlayerRef)!=0
			ssxRunnerQuest.log("Not possible to fill Player for " + ssxRunnerQuest.participantEdit[pos] + " because covered in cum", SSX_Runner_Quest.notifNotGoodActors())
			return none
		endIf
		if r.checkHasToWear(pos) && !ssxRunnerQuest.checkClothes(r, PlayerRef, pos)
			ssxRunnerQuest.log("Not possible to fill Player for " + ssxRunnerQuest.participantEdit[pos] + " because clothes", SSX_Runner_Quest.notifNotGoodActors())
			return none
		endIf
		if otherGender!=-1 ; Check if the other guy sex is compatible with player orientation
			int plSex = ssxRunnerQuest.calculateSex(PlayerRef)
			if ssxRunnerQuest.playerOrientation==0 ; SexLab sexuality
				int playerSLOr = ssxRunnerQuest.SexLab.GetSexuality(PlayerRef)
				if playerSLOr<36 ; homo
					if (plSex==0 && (otherGender==1 || otherGender==3)) || (plSex==1 && (otherGender==0 || otherGender==2)) ; partner different sex
						ssxRunnerQuest.log("Not possible to fill Player for " + ssxRunnerQuest.participantEdit[pos] + " because is homo and partner different sex", SSX_Runner_Quest.notifNotGoodActors())
						return none ; Not good
					endIf
				elseIf playerSLOr>65 ; hetero
					if (plSex==1 && (otherGender==1 || otherGender==3)) || (plSex==0 && (otherGender==0 || otherGender==2)) ; partner same sex
						ssxRunnerQuest.log("Not possible to fill Player for " + ssxRunnerQuest.participantEdit[pos] + " because is hetero and partner is same sex", SSX_Runner_Quest.notifNotGoodActors())
						return none ; Not good
					endIf 
				endIf
			elseIf ssxRunnerQuest.playerOrientation==1 ; Hetero Human
				if otherGender>1 ; Is a creature
					ssxRunnerQuest.log("Not possible to fill Player for " + ssxRunnerQuest.participantEdit[pos] + " because is hetero and partner is creature", SSX_Runner_Quest.notifNotGoodActors())
					return none
				endIf
				if (plSex==0 && otherGender!=1) || (plSex==1 && otherGender!=0)
					ssxRunnerQuest.log("Not possible to fill Player for " + ssxRunnerQuest.participantEdit[pos] + " because is hetero and partner has same sex", SSX_Runner_Quest.notifNotGoodActors())
					return none
				endIf
			elseIf ssxRunnerQuest.playerOrientation==2 ; Bisexual Human
				if otherGender>1 ; is a creature
					ssxRunnerQuest.log("Not possible to fill Player for " + ssxRunnerQuest.participantEdit[pos] + " because partner is creature", SSX_Runner_Quest.notifNotGoodActors())
					return none
				endIf
			elseIf ssxRunnerQuest.playerOrientation==3 ; Homo Human
				if otherGender>1 ; Is a creature
					ssxRunnerQuest.log("Not possible to fill Player for " + ssxRunnerQuest.participantEdit[pos] + " because partner is creature", SSX_Runner_Quest.notifNotGoodActors())
					return none
				endIf
				if (plSex==1 && otherGender!=1) || (plSex==0 && otherGender!=0)
					ssxRunnerQuest.log("Not possible to fill Player for " + ssxRunnerQuest.participantEdit[pos] + " because is homo and partner different sex", SSX_Runner_Quest.notifNotGoodActors())
					return none
				endIf
			elseIf ssxRunnerQuest.playerOrientation==4 ; Creatures
				if otherGender<2 ; not a creature
					ssxRunnerQuest.log("Not possible to fill Player for " + ssxRunnerQuest.participantEdit[pos] + " because partner is human", SSX_Runner_Quest.notifNotGoodActors())
					return none
				endIf
			elseIf ssxRunnerQuest.playerOrientation==5 ; Hetero H&C
				if (plSex==0 && (otherGender==0 || otherGender==2)) || (plSex==1 && (otherGender==1 || otherGender==3))
					ssxRunnerQuest.log("Not possible to fill Player for " + ssxRunnerQuest.participantEdit[pos] + " because is hetero and partner has same sex", SSX_Runner_Quest.notifNotGoodActors())
					return none
				endIf
			elseIf ssxRunnerQuest.playerOrientation==6 ; Bisex H&C
				; always good
			elseIf ssxRunnerQuest.playerOrientation==7 ; Homo H&C
				if (plSex==1 && (otherGender==0 || otherGender==2)) || (plSex==0 && (otherGender==1 || otherGender==3))
					ssxRunnerQuest.log("Not possible to fill Player for " + ssxRunnerQuest.participantEdit[pos] + " because is homo and partner different sex", SSX_Runner_Quest.notifNotGoodActors())
					return none
				endIf
			endIf
		endIf
		return PlayerRef
	elseIf r.isGenericWPl(pos) && !PlayerRef.isInFaction(ssxRunnerQuest.SexLabAnimatingFaction) && avoids.find(PlayerRef)==-1 && !PlayerRef.isInFaction(ssxRunnerQuest.ssxInvolved) && people.find(PlayerRef)!=-1
		if (ssxRunnerQuest.excludeActorsWithCum==0 || ssxRunnerQuest.SexLab.CountCum(PlayerRef)==0) && (!r.checkHasToWear(pos) || ssxRunnerQuest.checkClothes(r, PlayerRef, pos))
			if Utility.randomInt(0, len)==0
				if pos==0 ; Check if the other guy sex is compatible with player orientation
					int plSex = ssxRunnerQuest.calculateSex(PlayerRef)
					if ssxRunnerQuest.playerOrientation==0 ; SexLab sexuality
						int playerSLOr = ssxRunnerQuest.SexLab.GetSexuality(PlayerRef)
						if playerSLOr<36 ; homo
							if (plSex==1 && (otherGender==1 || otherGender==3)) || (plSex==0 && (otherGender==0 || otherGender==2)) ; partner different sex
								return PlayerRef
							endIf
						elseIf playerSLOr>65 ; hetero
							if (plSex==0 && (otherGender==1 || otherGender==3)) || (plSex==1 && (otherGender==0 || otherGender==2)) ; partner same sex
								return PlayerRef
							endIf
						endIf
					elseIf ssxRunnerQuest.playerOrientation==1 ; Hetero Human
						if otherGender<2 && ((plSex==1 && otherGender!=1) || (plSex==0 && otherGender!=0))
							return PlayerRef
						endIf
					elseIf ssxRunnerQuest.playerOrientation==2 ; Bisexual Human
						if otherGender<2 ; not a creature
							return PlayerRef
						endIf
					elseIf ssxRunnerQuest.playerOrientation==3 ; Homo Human
						if otherGender<2 && ((plSex==0 && otherGender!=1) || (plSex==1 && otherGender!=0))
							return PlayerRef
						endIf
					elseIf ssxRunnerQuest.playerOrientation==4 ; Creatures
						if otherGender>1 ; is a creature
							return PlayerRef
						endIf
					elseIf ssxRunnerQuest.playerOrientation==5 ; Hetero H&C
						if (plSex==1 && (otherGender==0 || otherGender==2)) || (plSex==0 && (otherGender==1 || otherGender==3))
							return PlayerRef
						endIf
					elseIf ssxRunnerQuest.playerOrientation==6 ; Bisex H&C
						; always good
					elseIf ssxRunnerQuest.playerOrientation==7 ; Homo H&C
						if (plSex==0 && (otherGender==0 || otherGender==2)) || (plSex==1 && (otherGender==1 || otherGender==3))
							return PlayerRef
						endIf
					endIf
				endIf
				return PlayerRef
			endIf
		endIf
	endIf

	; Do we need a specific race?
	if r.isSameRace(pos) && forTheRace
		String asRaceKey = sslCreatureAnimationSlots.GetRaceKey(forTheRace.GetRace())
		Actor someone = None
		int times = 0
		while times<10
			someone = people[Utility.randomInt(0, len - 1)]
			if ssxRunnerQuest.validateActor(someone) && avoids.find(someone)==-1 && sslCreatureAnimationSlots.GetRaceKey(someone.GetRace()) == asRaceKey && !someone.isInFaction(ssxRunnerQuest.ssxInvolved)
				return someone
			endIf
			someone = None
			times+=1
		endWhile
		; Find in linear mode
		times = len
		while times
			times-=1
			someone = people[times]
			if ssxRunnerQuest.validateActor(someone) && avoids.find(someone)==-1 && sslCreatureAnimationSlots.GetRaceKey(someone.GetRace()) == asRaceKey && !someone.isInFaction(ssxRunnerQuest.ssxInvolved)
				return someone
			endIf
		endWhile
		return None ; not found
	endIf
	
	while ssxRunnerQuest.letsWait
		Utility.waitMenuMode(0.5)
	endWhile

	; OK, we need to pick an actor. pos==1 anyone, pos==0 we should respect the sexual orientation preferences
	if otherGender==-1 ; Anyone
		int tries=0
		Actor someone=none
		while tries<10
			someone = people[Utility.randomInt(0, len - 1)]
			if ssxRunnerQuest.validateActor(someone) && (!r.checkHasToWear(pos) || ssxRunnerQuest.checkClothes(r, someone, pos)) && (ssxRunnerQuest.excludeActorsWithCum==0 || ssxRunnerQuest.SexLab.CountCum(someone)==0) && avoids.find(someone)==-1 && !someone.isInFaction(ssxRunnerQuest.ssxInvolved)
				if notReallyRequired || !someone.isInFaction(ssxRunnerQuest.SexLabAnimatingFaction)
					return someone
				endIf
			endIf
			tries+=1
		endWhile
		int i = len
		while i
			i-=1
			someone = people[i]
			if ssxRunnerQuest.validateActor(someone) && (!r.checkHasToWear(pos) || ssxRunnerQuest.checkClothes(r, someone, pos)) && (ssxRunnerQuest.excludeActorsWithCum==0 || ssxRunnerQuest.SexLab.CountCum(someone)==0) && avoids.find(someone)==-1 && !someone.isInFaction(ssxRunnerQuest.ssxInvolved)
				if notReallyRequired || !someone.isInFaction(ssxRunnerQuest.SexLabAnimatingFaction)
					return someone
				endIf
			endIf
		endWhile
	endIf
	
	; Find sombody compatile with the genders
	int tries=0
	Actor someone=none
	int otherMFSB = ssxRunnerQuest.getSexBitsForType(otherGender)
	while tries<10
		someone = people[Utility.randomInt(0, len - 1)]
		if ssxRunnerQuest.validateActor(someone) && (!r.checkHasToWear(pos) || ssxRunnerQuest.checkClothes(r, someone, pos)) && (ssxRunnerQuest.excludeActorsWithCum==0 || ssxRunnerQuest.SexLab.CountCum(someone)==0) && avoids.find(someone)==-1 && !someone.isInFaction(ssxRunnerQuest.ssxInvolved)
			if notReallyRequired || !someone.isInFaction(ssxRunnerQuest.SexLabAnimatingFaction)
				; Check the gender/sexual orientation
				int mfsb = ssxRunnerQuest.getMFSB(someone)
				if mfsb==0 && Math.LogicalAnd(otherMFSB, 1)==1 ; MH
					return someone
				elseIf mfsb==1 && Math.LogicalAnd(otherMFSB, 2)==2 ; FH
					return someone
				elseIf mfsb==2 && Math.LogicalAnd(otherMFSB, 4)==4 ; MC
					return someone
				elseIf mfsb==3 && Math.LogicalAnd(otherMFSB, 8)==8 ; FC
					return someone
				endIf
			endIf
		endIf
		tries+=1
	endWhile
	int i = len
	while i
		i-=1
		someone = people[i]
		if ssxRunnerQuest.validateActor(someone) && (!r.checkHasToWear(pos) || ssxRunnerQuest.checkClothes(r, someone, pos)) && (ssxRunnerQuest.excludeActorsWithCum==0 || ssxRunnerQuest.SexLab.CountCum(someone)==0) && avoids.find(someone)==-1 && !someone.isInFaction(ssxRunnerQuest.ssxInvolved)
			if notReallyRequired || !someone.isInFaction(ssxRunnerQuest.SexLabAnimatingFaction)
				; Check the gender/sexual orientation
				int mfsb = ssxRunnerQuest.getMFSB(someone)
				if mfsb==0 && Math.LogicalAnd(otherMFSB, 1)==1 ; MH
					return someone
				elseIf mfsb==1 && Math.LogicalAnd(otherMFSB, 2)==2 ; FH
					return someone
				elseIf mfsb==2 && Math.LogicalAnd(otherMFSB, 4)==4 ; MC
					return someone
				elseIf mfsb==3 && Math.LogicalAnd(otherMFSB, 8)==8 ; FC
					return someone
				endIf
			endIf
		endIf
	endWhile
	if otherGender==-1
		ssxRunnerQuest.log("Not possible to find a " + ssxRunnerQuest.participantEdit[pos], SSX_Runner_Quest.notifNotGoodActors())
	elseIf ssxRunnerQuest.nexusMode==1
		ssxRunnerQuest.log("Not possible to find a " + ssxRunnerQuest.participantEdit[pos] + " otherGender=" + otherGender + " possible values=" + ssxRunnerQuest.orientationModes[ssxRunnerQuest.getSexBitsForType(otherGender)], SSX_Runner_Quest.notifNotGoodActors())
	else
		ssxRunnerQuest.log("Not possible to find a " + ssxRunnerQuest.participantEdit[pos] + " otherGender=" + otherGender + " possible values=" + ssxRunnerQuest.orientationModesC[ssxRunnerQuest.getSexBitsForType(otherGender)], SSX_Runner_Quest.notifNotGoodActors())
	endIf
	return none ; Nobody was found
endFunction





Function startRuleByPlayerAsking(Actor target, int request, int answer)
	{Check for all rules where the player ask, where this guy can be in, check also for the ask dialogue}
	; And then start the scene by usign the specific scene slot
	SSX_Rule_Alias[] setOfRules = new SSX_Rule_Alias[8]
	SSX_Rule_Alias[] validRules = new SSX_Rule_Alias[8]
	int num = ssxRunnerQuest.rules.length
	int numPossibleRules = 0
	int mode = 1
	SSX_Rule_Alias r = None
	while num
		num-=1
		r = ssxRunnerQuest.rules[num]
		if r.startsOn==1 && (r.validDst.find(target)!=-1 || r.validSrc.find(target)!=-1 || r.validColl.find(target)!=-1)
			if (r.getDialogue(0)==request)
				if (answer==-1 || r.getDialogue(1)==answer)
					setOfRules[numPossibleRules] = r
					numPossibleRules+=1
				else
					if ssxRunnerQuest.ssxNextAnswerWillBe0==1 || ssxRunnerQuest.ssxNextAnswerWillBe0==2
						; A consequence can be: 0=No one, no sex, 1=consensual sex, 2=rape, 3=masturbation
						setOfRules[numPossibleRules] = r
						numPossibleRules+=1
						mode = 11
					endIf
				endIf
			endIf
		endIf
	endWhile
	
	if numPossibleRules==0
		if target
			ssxRunnerQuest.log("Not a single rule that has the player asking is found with " + target.getDisplayName(), SSX_Runner_Quest.notifErrors())
		else
			ssxRunnerQuest.log("Not a single rule that has the player asking is found, the other actor is missing", SSX_Runner_Quest.notifErrors())
		endIf
		return
	endIf
	
	; Now pick one of the possible ones
	r = setOfRules[Utility.randomInt(0, numPossibleRules - 1)]
	; And fill it
	if fillTheRule(r, mode, target)
		return ; Rule should be started
	endIf
	while numPossibleRules
		numPossibleRules-=1
		r = setOfRules[numPossibleRules]
		if fillTheRule(r, mode, target)
			return ; Rule should be started
		endIf
	endWhile
	ssxRunnerQuest.log("Not a single rule can start by player asking.", SSX_Runner_Quest.notifWarnings())
endFunction




