Scriptname SSX_SPID_Scanner_Quest extends Quest ; V1.4e

import PO3_SKSEFunctions

; ((- Properties
SSX_Runner_Quest Property ssxRunnerQuest Auto
Actor Property PlayerRef Auto
SexLabFramework Property SexLab Auto
Faction Property ssxScanningFaction Auto
Faction Property SexLabAnimatingFaction Auto

Spell Property ssxRegisterActorSpell Auto
MagicEffect Property ssxRegisterActorEffect Auto

ObjectReference[] Property beds Auto
int Property numBeds Auto
bool iAmScanning
Actor [] badActors
int badNum = 0
bool shuttingDown ; True if (active state) OnEndState is running
; -))

Function doInit()
	RegisterForModEvent("ssxStartSSXMod", "startScannerEvent")
	RegisterForModEvent("ssxStopSSXMod", "stopScannerEvent")
	RegisterForModEvent("ssxStartScanner", "startScannerEvent")
	RegisterForModEvent("ssxStopScanner", "stopScannerEvent")
	RegisterForModEvent("ssxScanCell", "scanNow")
	ssxRunnerQuest.log("SPID Scanner events registered", SSX_Runner_Quest.notifScanDetails())
	iAmScanning=false
	badActors = new Actor[100]
EndFunction

Function recoverFromEmptyState()
	{This could theoretically happen if someone installs SSXNG over SSX}
	ssxRunnerQuest.log("SPID Scanner has reached a state that should not be reachable. Trying to recover...", SSX_Runner_Quest.notifScanDetails())
	goToState("inactive")
EndFunction

; ((- Events have to be defined in the empty state
Event startScannerEvent(int mode)
	recoverFromEmptyState()
EndEvent

Event stopScannerEvent(String unused)
	recoverFromEmptyState()
EndEvent

Event scanNow(String mod)
	if mod=="CellScanner" && ssxRunnerQuest.iAmActive && !iAmScanning
		UnregisterForUpdate()
		RegisterForSingleUpdate(2.0)
		goToState("active")
	endIf
EndEvent

Function cleanup()
	while badNum
		badNum -= 1
		badActors[badNum].removeFromFaction(ssxScanningFaction)
		badActors[badNum] = none
	endWhile
	ssxRunnerQuest.removeFromScanningFaction()
	ssxRunnerQuest.schedulerIAmDone("Scanner")
	iAmScanning=false
EndFunction

bool Function reserveSchedule()
	int s = ssxRunnerQuest.schedulerCanIDo("Scanner")
	while s
		if s == 2
			return false
		endIf ; s == 1
		Utility.waitMenuMode(0.5)
		s = ssxRunnerQuest.schedulerCanIDo("Scanner")
	endWhile ; s == 0
	cleanup()
	gotoState("inactive")
	return true
EndFunction


; -))


auto state inactive
	Function OnBeginState()
		ssxRunnerQuest.log("SPID Scanner going to inactive state", SSX_Runner_Quest.notifScanDetails())
		iAmScanning=false
	EndFunction
	
	Event startScannerEvent(int mode)
		; We start in the inactive state but also listen for the startScannerEvent 
		if mode>=1
			goToState("active")
		endIf
	EndEvent

	Event stopScannerEvent(String unused)
		; Ignore
	EndEvent
endState

state active
	Function OnBeginState()
		ssxRunnerQuest.log("SPID Scanner going to active state", SSX_Runner_Quest.notifScanDetails())
		int scanningDistance;
		if ssxRunnerQuest.scanMode==1
			scanningDistance = 2048
		elseIf ssxRunnerQuest.scanMode==2
			scanningDistance = 3584
		elseIf ssxRunnerQuest.scanMode==3
			scanningDistance = 5120
		else
			GoToState("inactive")
			return
		endIf
		
		String[] conditionList = new String[1]
		conditionList[0] = "Subject | GetDistance | 00000014 ~ Skyrim.esm | NONE | < | " + scanningDistance + " | AND"
		PO3_SKSEFunctions.AddMagicEffectToSpell(ssxRegisterActorSpell, ssxRegisterActorEffect, 0.0, 0, 0, 0.0, conditionList)
		ssxRunnerQuest.updateCellAndLocation(PlayerRef.getParentCell())
		RegisterForSingleUpdate(5.0)
	EndFunction
	
	Function OnEndState()
		shuttingDown = true
		; This is guaranteed to execute completely before inactive state begins, which is important 
		PO3_SKSEFunctions.removeMagicEffectFromSpell(ssxRegisterActorSpell, ssxRegisterActorEffect, 0.0, 0, 0, 0.0)
		ssxRunnerQuest.log("SPID Scanner spell has " + ssxRegisterActorSpell.getNumEffects() + " effects", SSX_Runner_Quest.notifScanDetails())
		
		UnregisterForUpdate()
		Utility.waitMenuMode(1.0)
		cleanup()
		shuttingDown = false
	EndFunction

	Event startScannerEvent(int mode)
		if mode<1
			GoToState("inactive")
		elseIf mode>=1
			; this avoids a race condition when stopping and starting the scanner quickly like the MCM does
			while shuttingDown
				Utility.waitMenuMode(1.0)
			endWhile
			GoToState("active")
		endIf
	EndEvent

	Event stopScannerEvent(String unused)
		GoToState("inactive")
	EndEvent

	Event OnUpdate()
		iAmScanning=true
		
		if reserveSchedule()
			return
		endIf
		ssxRunnerQuest.clearCached() ; wipe all cached data for actors
				
		ssxRunnerQuest.log("SPID Scanner Maintenance of actors", SSX_Runner_Quest.notifScanDetails())
		ssxRunnerQuest.playerSex = 0
		if (ssxRunnerQuest.genderMode==0 && PlayerRef.getLeveledActorBase().getSex()) || (ssxRunnerQuest.genderMode!=0 && Math.LogicalAnd(SexLab.getGender(PlayerRef), 1))
			ssxRunnerQuest.playerSex = 1
		endIf

		if reserveSchedule()
			return
		endIf
		
		; remove actors that are no longer valid
		ssxRunnerQuest.removeNoMoreValidActors()
	
		if reserveSchedule()
			return
		endIf

		while ssxRunnerQuest.letsWait
			Utility.waitMenuMode(1.0)
		endWhile
		
		; Update the ssxRunnerQuest property, but check if at least two guys are required
		if ssxRunnerQuest.needP && (ssxRunnerQuest.needM || ssxRunnerQuest.needF || ssxRunnerQuest.needC || ssxRunnerQuest.needH || ssxRunnerQuest.needW || ssxRunnerQuest.needN) && ssxRunnerQuest.noActors()
			; Not enough actors
			while ssxRunnerQuest.letsWait
				Utility.waitMenuMode(1.0)
			endWhile
			RegisterForSingleUpdate(ssxRunnerQuest.rescanCellTime)
			ssxRunnerQuest.log("SPID Scanner - Scan completed. Not enough actors", SSX_Runner_Quest.notifScanDetails())
			cleanup()
			return
		endIf
		
		if reserveSchedule()
			return
		endIf

		while ssxRunnerQuest.letsWait
			Utility.waitMenuMode(1.0)
		endWhile
		
		; Find beds
		numBeds = ssxRunnerQuest.scanBeds(beds)
		ssxRunnerQuest.setBeds(beds, numBeds)

		if reserveSchedule()
			return
		endIf
		
		if ssxRunnerQuest.getDebug(SSX_Runner_Quest.notifActorsFoundLong())
			ssxRunnerQuest.dumpAllActors("SPID Scan")
		endIf
		
		; then wait for the next cycle
		RegisterForSingleUpdate(ssxRunnerQuest.rescanCellTime)
		ssxRunnerQuest.log("SPID Scanner - Scan completed. Found " + ssxRunnerQuest.countActors() + " actors", SSX_Runner_Quest.notifScanDetails())
		cleanup()
		
		; Restart the Rule Checker in case this was out first scan
		if !ssxRunnerQuest.noActors() && ssxRunnerQuest.checkingMode==0
			int handle = ModEvent.Create("ssxStartSSXRC")
			ModEvent.PushInt(handle, 0)
			ModEvent.Send(handle)
		elseIf ssxRunnerQuest.checkingMode==1
			int handle = ModEvent.Create("ssxDoIt")
			ModEvent.PushString(handle, "RuleChecker")
			ModEvent.Send(handle)
		endIf
	EndEvent
endState


Function addMe(Actor a)
	if ssxRunnerQuest.checkValidityForScan(a)
		ssxRunnerQuest.addToAllActors(a)
	else
		if badNum > 99
			badNum = 99
		endIf
		badActors[badNum] = a
		badNum += 1						
		If !a.isInFaction(SexLabAnimatingFaction)
			int n = ssxRunnerQuest.searchScene(none, -1, a)
			if n!=-1
				ssxRunnerQuest.log("## Release scene with " + a.getDisplayName(), SSX_Runner_Quest.notifScanDetails())
				ssxRunnerQuest.releaseScene(none, n, none)
			endIf
		endIf
	endIf
	if !a.IsInFaction(ssxRunnerQuest.ssxScanningFaction)
		a.addToFaction(ssxRunnerQuest.ssxScanningFaction)
	endIf
EndFunction
